import * as vscode from "vscode";
import { GitService, GitCommit } from "./GitService";
import path from "path";
import { CommitItem } from "./commit-tree.item";
import { EventBus } from "./event-bus";

export class CommitListProvider implements vscode.TreeDataProvider<CommitItem> {
  private state:string = "list";
  private filter: any = {};
  private _commits: GitCommit[] = [];
  private _onDidChangeTreeData: vscode.EventEmitter<CommitItem | undefined | void> = new vscode.EventEmitter<CommitItem | undefined | void>();
  readonly onDidChangeTreeData: vscode.Event<CommitItem | undefined | void> = this._onDidChangeTreeData.event;

  private _showFullPath: boolean = false;

  constructor(private readonly _gitService: GitService,private event: EventBus) {
    this.event.on("filter",(filter)=>{
      this.filter = filter;
      this.refresh();
    });
    this.event.on("refresh",()=>{
      this._makeCommitItems();
      this._onDidChangeTreeData.fire();
    });
  }

  _makeCommitItems() {

  }

  refresh(): void {
    this._onDidChangeTreeData.fire();
  }

  toggleFullPath(): void {
    this._showFullPath = !this._showFullPath;
    this.refresh();
  }

  getTreeItem(element: CommitItem): vscode.TreeItem {
    return element;
  }

  async getChildren(element?: CommitItem): Promise<CommitItem[]> {
    if (!element) {
      const commits = await this._gitService.getCommits("");
      return commits.map(commit => new CommitItem(`${commit.msg} -- ${commit.author}@${commit.hash} ${commit.date}`, vscode.TreeItemCollapsibleState.Expanded, commit.files));
    }
    if (element.files) {
      return element.files.map(file => new CommitItem(this._showFullPath ? file : path.basename(file), vscode.TreeItemCollapsibleState.None, undefined, file));
    }
    return [];
  }
}