import * as vscode from "vscode";
import * as path from "path";


// interface PathItem {
//     fpath?: string;
//     label: string;
//     children?: PathTree;
// }
// interface PathTree {
//     [key: string]: PathItem;
// }
// function makePathTree(paths: string[]): PathTree{
//     const pathTree: PathTree = {};
//     for (const file of paths) {
//         const pathParts = file.split("/");
//         if(pathParts.length === 1){
//             pathTree[pathParts[0]] = {fpath: file, label: pathParts[0]};
//             continue;
//         }
//         let currentNode = pathTree;
//         for (let i = 0; i < pathParts.length; i++) {
//             const pathItem = pathParts[i];
//             if(!currentNode.chilren){
//                 currentNode.chilren = {};
//             }
//             if (!currentNode.chilren[pathItem]) {
//                 currentNode[pathItem] = {
//                     label: pathItem,
//                     children: i !== pathParts.length - 1 ? {} : undefined,
//                     fpath: i === pathParts.length - 1 ? file : undefined,
//                 };
//             }  else
//             if(i !== pathParts.length - 1) {
//                  currentNode = currentNode[pathItem].children;
//             }
           
//         }
//     }
//     return {};
// }

export class CommitItem extends vscode.TreeItem {
    public iconPath?: string | vscode.IconPath | undefined = 'file';
    public children?: CommitItem[];
    constructor(
        public readonly label:string, 
        public state : vscode.TreeItemCollapsibleState, 
        public readonly commitHash: string,
        public readonly style:string,
        public readonly fpath?: string,
        public readonly files?: string[]
    ){
        super(label, state);
        if(this.isFileItem()){
            this.tooltip = this.fpath;
        }
        if(this.isCommitItem()) {
            this.iconPath = 'git-commit';
        }
    }
    makeChildren() {
        if(this.isCommitItem()) {
            if(this.style === "list"){
                this.children = this.files?.map(file => {
                    return new CommitItem(path.basename(file), vscode.TreeItemCollapsibleState.None, this.commitHash, "list", file);
                });
            } else {
                if(!this.files) {
                    return
                }
                for(const file of this.files) {
                    const fileParts = file.split("/");
                    if(fileParts.length === 1){
                        this.addChildren(new CommitItem(fileParts[0], vscode.TreeItemCollapsibleState.None, this.commitHash, "tree", file));
                        continue;
                    }
                    let currentNode: CommitItem = this;
                    for (let i = 0; i < fileParts.length; i++) {
                        const pathItem = fileParts[i];
                        const child = currentNode.getChildItem(pathItem);
                        if(!child) {
                            const newChild = new CommitItem(pathItem, vscode.TreeItemCollapsibleState.Collapsed, this.commitHash, "tree");
                            currentNode.addChildren(newChild);
                            currentNode = newChild;
                        } else {
                            currentNode = child;
                        }
                    }
                }
            }
        }
    }

    isCommitItem() {
        return this.files && !this.fpath;
    }

    isFileItem() {
        return this.fpath && this.children?.length === 0;
    }
    isFolderPath() {
        return this.fpath && this.children && this.children?.length > 0;       
    }
    addChildren(item: CommitItem) {
        this.iconPath = 'folder';
        const index = this.children?.findIndex(child => child.label === item.label);
        if(index === -1) {
            this.children?.push(item);
        }
    }  
    getChildItem(label: string) {
        return this.children?.find(child => child.label === label);
    } 
}